package com.asreader.p3xu;

import java.util.ArrayList;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.graphics.Color;
import android.Manifest;
import android.os.Bundle;
import android.os.Build;
import android.os.Handler;
import android.os.PowerManager;

//import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.View.OnTouchListener;
import android.view.Window;
import android.widget.Button;
import android.widget.ImageView;

import android.widget.TextView;
import android.widget.Toast;

import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

import com.asreader.R;

import com.asreader.p3xu.rfid.type.AsReaderP3xUCommandType;
import com.asreader.p3xu.rfid.type.AsReaderP3xULbtItem;
import com.asreader.p3xu.view.WriteMemoryActivity;
import com.asreader.p3xu.view.InventoryActivity;
import com.asreader.p3xu.view.OptionActivity;
import com.asreader.p3xu.view.ReadMemoryActivity;
import com.asreader.p3xu.view.TagAccessActivity;
import com.asreader.p3xu.view.UpdateFirmwareActivity;

import com.asreader.p3xu.device.AsReaderP3xUDevice;
import com.asreader.p3xu.device.AsReaderP3xUDeviceUsbCdc;
import com.asreader.p3xu.device.type.AsReaderP3xUConnectionState;
import com.asreader.p3xu.device.type.AsReaderP3xUKeyState;
import com.asreader.p3xu.device.type.AsReaderP3xUKeyType;
import com.asreader.p3xu.diagnostics.AsReaderP3xUASRLog;
import com.asreader.p3xu.rfid.event.AsReaderP3xUEventListener;
import com.asreader.p3xu.rfid.type.AsReaderP3xUActionState;
import com.asreader.p3xu.rfid.type.AsReaderP3xUResultCode;

import com.asreader.support.andro.app.rfid.dialog.WaitDialog;
import com.asreader.support.andro.app.rfid.util.UserConfigs;
import com.asreader.support.andro.app.rfid.view.base.AccessActivity;

public class MainActivity extends Activity implements OnClickListener, OnTouchListener, AsReaderP3xUEventListener {

    private static final String TAG = MainActivity.class.getSimpleName();
    private static final String ACTION_USB_PERMISSION = "com.android.example.USB_PERMISSION";

    private static boolean WRITE_LOG = true;
    private static final String LOG_PATH = "Log";
    private static final String LOG_PREFIX = "AsReaderP3xU_Demo";

    public static final String APP_NAME = "rfid.reader.demo";
    private static final int DEFAULT_BATTERY_INTERVAL = 10;
    private static final int VIEW_INVENTORY = 1000;
    private static final int VIEW_OPTION = 1006;
    private static final int VIEW_DFU = 1007;
    private static final int RESULT_DISCONNECTED = Activity.RESULT_FIRST_USER + 1;

    private static final int REQUEST_PERMISSION_CONTACTS = 1000;

    private PowerManager.WakeLock wakeLock = null;
    private AsReaderP3xU mAsReaderP3xU = null;

    private TextView txtDemoVersion;
    private TextView txtSDKVersion;
    private TextView txtFirmwareVersion;
    private TextView txtHardwareVersion;
    private TextView txtAddress;
    private TextView txtGlobalBand;
    private TextView txtSerialNumber;
    private Button btnInventory;
    private Button btnReadMemory;
    private Button btnWriteMemory;
    private Button btnTagAccess;
    private Button btnOption;
    private Button btUpdateFirmware;
    private ImageView imgLogo;

    private MenuItem mnuConnectUsbDevice;
    private MenuItem mnuDisconnectDevice;

    private Handler mHandler;
    private AsReaderP3xUDevice mDevice = null;

    private final int COLOR_ENABLED = Color.BLACK;
    private final int COLOR_DISABLED = Color.LTGRAY;

    private String[] mPermissions = null;
    public static UserConfigs mUserConfigs = null;

    private boolean isFinishedDiscovery;
    private boolean isBleDevConecting;

    private String firmwareVersionStr;

    //USB
    private static AsReaderP3xUDeviceUsbCdc usbCdc;

    @SuppressWarnings("deprecation")
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        usbCdc = new AsReaderP3xUDeviceUsbCdc(this);

        requestWindowFeature(Window.FEATURE_INDETERMINATE_PROGRESS);

        getActionBar().setDisplayShowHomeEnabled(true); // zsj 2021-02-25 隐藏ActionBar左边icon
        setContentView(R.layout.activity_main);

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
            mPermissions = new String[]{Manifest.permission.WAKE_LOCK,Manifest.permission.READ_MEDIA_IMAGES,Manifest.permission.READ_MEDIA_VIDEO};
        } else {
            mPermissions = new String[]{Manifest.permission.WAKE_LOCK, Manifest.permission.WRITE_EXTERNAL_STORAGE,
                    Manifest.permission.READ_EXTERNAL_STORAGE};
        }

        mUserConfigs = UserConfigs.getInstance(this.getApplicationContext());

        setProgressBarIndeterminateVisibility(true);
        // Initialize Widget
        initWidgets();

        // Disable butotns
        enableMenuButtons(false);

        checkPermission();
    }

    // Visible Menu Item
    private void visibleMenuItem(AsReaderP3xUConnectionState state) {
        if (state == AsReaderP3xUConnectionState.Connected){
            if (mnuConnectUsbDevice != null)
                mnuConnectUsbDevice.setVisible(false);
            if (mnuDisconnectDevice != null)
                mnuDisconnectDevice.setVisible(true);
        }else {
            if (mnuConnectUsbDevice != null)
                mnuConnectUsbDevice.setVisible(true);
            if (mnuDisconnectDevice != null)
                mnuDisconnectDevice.setVisible(false);
        }
    }

    private boolean checkPermission() {
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.M) {
            onPermissionsResult(true);
            return false;
        }

        ArrayList<String> permissions = new ArrayList<String>();
        int grantResult = PackageManager.PERMISSION_GRANTED;

        for (int i = 0; i < mPermissions.length; i++) {
            grantResult = PackageManager.PERMISSION_GRANTED;
            grantResult = ContextCompat.checkSelfPermission(this, mPermissions[i]);
            if (grantResult == PackageManager.PERMISSION_DENIED) {
                permissions.add(mPermissions[i]);
            }
        }
        if (permissions.size() <= 0) {
            onPermissionsResult(true);
            return false;
        }

        String[] requestPermissions = new String[permissions.size()];
        permissions.toArray(requestPermissions);
        ActivityCompat.requestPermissions(this, permissions.toArray(requestPermissions), REQUEST_PERMISSION_CONTACTS);

        AsReaderP3xUASRLog.e(TAG, "INFO. checkPermission()");
        return true;
    }

    private final int ACCESS_LOCATION = 1;

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.main, menu);
        return true;
    }
    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {

        if (mAsReaderP3xU != null) {
            mnuConnectUsbDevice = menu.findItem(R.id.menu_connect_device_usb);
            mnuDisconnectDevice = menu.findItem(R.id.menu_disconnect);

            visibleMenuItem(mAsReaderP3xU.getState());
        }
        return super.onPrepareOptionsMenu(menu);
    }
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        int id = item.getItemId();

        switch (id) {
            case R.id.menu_connect_device_usb:
                mDevice = AsReaderP3xUManager.getInstance().getCurrentDevice();
                if (mDevice == null) {
                    mDevice = usbCdc;
                    if (mAsReaderP3xU != null) {
                        mAsReaderP3xU.connectDevice(mDevice);
                    }
                } else {
                    onStateChanged(mAsReaderP3xU, AsReaderP3xUConnectionState.Connected);
                }
                return true;
            case R.id.menu_disconnect:
                WaitDialog.show(this, R.string.disconnecting_AsReaderP3xU);
                mAsReaderP3xU.disconnectDevice();
                return true;
        }
        return super.onOptionsItemSelected(item);
    }
    @Override
    public void onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {
        if (requestCode != REQUEST_PERMISSION_CONTACTS)
            return;

        boolean result = true;
        for (int i = 0; i < grantResults.length; i++) {
            result &= grantResults[i] != PackageManager.PERMISSION_DENIED;
        }

        onPermissionsResult(result);
    }

    private void onPermissionsResult(boolean result) {
        if (!result) {
            AsReaderP3xUASRLog.e(TAG, "Failed to get permissions");
            setProgressBarIndeterminateVisibility(false);
            return;
        }

        if (WRITE_LOG)
            AsReaderP3xUASRLog.startUp(LOG_PATH, LOG_PREFIX);

        // Using higher levels can cause serious performance degradation.
        AsReaderP3xUASRLog.setLogLevel(AsReaderP3xUASRLog.ERROR);

        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onCreate()");

        // Setup Always Wakeup
        setupAlwaysWakeup();

        // Initialize RFID Reader
        initAsReaderP3xU();

        // Display Demo Version
        displayVersion();

        // Display AsReaderP3xUSDK Version
        displaySDKVerserion();

        setProgressBarIndeterminateVisibility(false);
    }

    @Override
    protected void onPause() {
        if (mAsReaderP3xU != null) {
            AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onPause()");
        }
        super.onPause();
    }

    @Override
    protected void onResume() {
        // Load Configuration
        loadConfig();

        if (mAsReaderP3xU != null) {
            mAsReaderP3xU.setEventListener(this);

            if (mAsReaderP3xU.getState() == AsReaderP3xUConnectionState.Connected) {
            }else  {
                setDisconnectedState();
            }
        }
        super.onResume();
    }

    @Override
    protected void onStop() {
        if (mAsReaderP3xU != null) {
            AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onStop()");
        }
        super.onStop();
    }

    @Override
    protected void onDestroy() {
        if (mAsReaderP3xU != null) {
            AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onDestroy()");

            // Destroy Rfid Reader
            AsReaderP3xUManager.onDestroy();

            // Destroy USB device
            if (mDevice != null) {
                mDevice.destroy();
                mDevice = null;
            }

            defaultWakeup();

            AsReaderP3xUASRLog.shutdown();
        }
        super.onDestroy();
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {

        AsReaderP3xUASRLog.d(TAG, AsReaderP3xUASRLog.INFO, "DEBUG. onActivityResult(%s, %s)", getRequestCode(requestCode), getResultCode(resultCode));

        switch (requestCode) {
            case VIEW_INVENTORY:
            case AccessActivity.VIEW_READ_MEMORY:
            case AccessActivity.VIEW_WRITE_MEMORY:
            case AccessActivity.VIEW_TAG_ACCESS:
            case VIEW_OPTION:
            case VIEW_DFU:
                switch (resultCode) {
                    case RESULT_DISCONNECTED:
                        setDisconnectedState();
                        break;
                    case RESULT_OK:
                        break;
                }
                break;
        }

        super.onActivityResult(requestCode, resultCode, data);
    }

    @Override
    public void onClick(View v) {
        Intent intent;
        int id = v.getId();

        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onClick(%s)", getOnClickId(id));

        switch (id) {
            case R.id.inventory:
                intent = new Intent(this, InventoryActivity.class);
                startActivityForResult(intent, VIEW_INVENTORY);
                break;
            case R.id.read_memory:
                intent = new Intent(this, ReadMemoryActivity.class);
                startActivityForResult(intent, AccessActivity.VIEW_READ_MEMORY);
                break;
            case R.id.write_memory:
                intent = new Intent(this, WriteMemoryActivity.class);
                startActivityForResult(intent, AccessActivity.VIEW_WRITE_MEMORY);
                break;
            case R.id.tag_access:
                intent = new Intent(this, TagAccessActivity.class);
                startActivityForResult(intent, AccessActivity.VIEW_TAG_ACCESS);
                break;
            case R.id.option:
                intent = new Intent(this, OptionActivity.class);
                startActivityForResult(intent, VIEW_OPTION);
                break;
            case R.id.updatefirmware:
                intent = new Intent(this, UpdateFirmwareActivity.class);
                intent.putExtra("CurrentFirmWareVersion", firmwareVersionStr);
//                startActivity(intent);
                startActivityForResult(intent, VIEW_OPTION);
            default:
                break;
        }
    }

    private static boolean stateChanging;
    @SuppressWarnings("deprecation")
    @Override
    public void onStateChanged(AsReaderP3xU asReaderP3xU, final AsReaderP3xUConnectionState state) {
        closeOptionsMenu();
        switch (state) {
            case Disconnected:
                if (mAsReaderP3xU.getResultCode() == AsReaderP3xUResultCode.NotSupportFirmware) {
                    AlertDialog.Builder alert = new AlertDialog.Builder(this);
                    alert.setTitle(R.string.system_alert);
                    alert.setIcon(android.R.drawable.ic_dialog_alert);
                    alert.setMessage(R.string.not_support_firmware);
                    alert.setPositiveButton(R.string.action_ok, null);
                    alert.show();
                }
                setDisconnectedState();
                setProgressBarIndeterminateVisibility(false);
                break;
            case Connecting:
                setProgressBarIndeterminateVisibility(true);
                enableMenuButtons(false);
                imgLogo.setImageResource(R.drawable.asreader_log_off);
                break;
            case Listen:
                enableMenuButtons(false);
                imgLogo.setImageResource(R.drawable.asreader_log_off);
                break;
            case Connected:
                if (stateChanging) return;
                stateChanging = true;
                setProgressBarIndeterminateVisibility(false);
                WaitDialog.show(this, R.string.load_option);
                new Thread(new Runnable() {
                    @Override
                    public void run() {
                        String versionName = "";
                        String hardVersion = "";
                        String serialNumber = "";
                        String rfModuleVersion = "";
                        String globalBand = "";
                        try {
                            versionName = mAsReaderP3xU.getFirmwareVersion();
                            hardVersion = mAsReaderP3xU.getHardwareVersion();
                            serialNumber = mAsReaderP3xU.getSerialNumber();
                            rfModuleVersion = mAsReaderP3xU.getRFModuleVersion();
                            globalBand = mAsReaderP3xU.getRegion();
                            if (versionName == null || versionName.equals(""))
                                versionName = mAsReaderP3xU.getFirmwareVersion();
                            if (serialNumber == null || serialNumber.equals(""))
                                serialNumber = mAsReaderP3xU.getSerialNumber();
                            if (rfModuleVersion == null || rfModuleVersion.equals(""))
                                rfModuleVersion = mAsReaderP3xU.getRFModuleVersion();
                            if (globalBand == null || globalBand.equals(""))
                                globalBand = mAsReaderP3xU.getRegion();
                            if (!mUserConfigs.Load(mAsReaderP3xU)) {
                                synchronized (this) {
                                    stateChanging = false;
                                }
                                return;
                            }



                            mAsReaderP3xU.setBuzzer(mUserConfigs.getVolume());
                            String devieModel = mAsReaderP3xU.getDeviceModel();
                            if (devieModel.equals("0")) {
                                mAsReaderP3xU.setInventoryTime(mUserConfigs.getInventoryTime());
                                mAsReaderP3xU.setIdleTime(mUserConfigs.getIdleTime());
                            }
                            mAsReaderP3xU.setQValue(mUserConfigs.getQValue());
                            mAsReaderP3xU.setFrequencyAutomatic(mUserConfigs.getFrequencyAuto());
                            if (mUserConfigs.getFrequencyAuto() == false) {
                                AsReaderP3xULbtItem[] mFreqTable = mAsReaderP3xU.getLbt();
                                if (mFreqTable != null) {
                                    String saveFrequency = mUserConfigs.getFrequency();
                                    for (int i = 0; i < mFreqTable.length; i++) {
                                        AsReaderP3xULbtItem item = mFreqTable[i];
                                        if (item.getFrequency().equals(saveFrequency)) {
                                            item.setUsed(true);
                                        }else{
                                            item.setUsed(false);
                                        }
                                    }
                                    mAsReaderP3xU.setLbt(mFreqTable);
                                }
                            }

                            mAsReaderP3xU.clearEpcMask();
                            for (int i = 0; i < mUserConfigs.getEpcMaskCount(); i++) {
                                mAsReaderP3xU.addEpcMask(mUserConfigs.getEpcMask(i));
                            }
                        } catch (Exception ae) {
                            synchronized (this) {
                                stateChanging = false;
                            }
                            AsReaderP3xUASRLog.e(TAG, "ERROR. Connected Thread - exit by exception.");
                            return;
                        }

                        if (mAsReaderP3xU.getState() != AsReaderP3xUConnectionState.Connected) {
                            synchronized (this) {
                                stateChanging = false;
                            }
                            AsReaderP3xUASRLog.e(TAG, "ERROR. Connected Thread - exit thread. state is not connected.");
                            return;
                        }
                        synchronized (this) {
                            stateChanging = false;
                        }

                        runOnUiThread(new InitRunnable(versionName, hardVersion, serialNumber, rfModuleVersion, globalBand));
                    }
                }).start();
                break;
            default:
                break;

        }
    }


    @Override
    public void onActionChanged(AsReaderP3xU asReaderP3xU, AsReaderP3xUActionState action) {
        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "EVENT. onActionChanged(%s)", action);
    }

    @Override
    public void onReadTag(AsReaderP3xU asReaderP3xU, AsReaderP3xUActionState action, String tag, float rssi) {
        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "EVENT. onReadTag(%s, [%s], %.2f)", action, tag, rssi);
    }

    @Override
    public void onAccessResult(AsReaderP3xU asReaderP3xU, AsReaderP3xUResultCode code, AsReaderP3xUActionState action, String epc, String data) {
        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "EVENT. onAccessResult(%s, %s, [%s], [%s])", code, action, epc, data);
    }

    @Override
    public void onKeyEvent(AsReaderP3xU asReaderP3xU, AsReaderP3xUKeyType key, AsReaderP3xUKeyState state) {
        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "EVENT. onKeyEvent [%s, %s]", key, state);
    }

    // Initialize Widgets
    private void initWidgets() {

        txtDemoVersion = (TextView) findViewById(R.id.demo_version);
        txtSDKVersion = (TextView) findViewById(R.id.SDK_version);
        txtFirmwareVersion = (TextView) findViewById(R.id.firmware_version);
        txtHardwareVersion = (TextView) findViewById(R.id.hardware_version);
        txtAddress = (TextView) findViewById(R.id.address);
        txtGlobalBand = (TextView) findViewById(R.id.global_band);
        txtSerialNumber = (TextView) findViewById(R.id.serial_number);
        btnInventory = (Button) findViewById(R.id.inventory);
        btnInventory.setOnClickListener(this);
        btnInventory.setOnTouchListener(this);
        btnReadMemory = (Button) findViewById(R.id.read_memory);
        btnReadMemory.setOnClickListener(this);
        btnReadMemory.setOnTouchListener(this);
        btnWriteMemory = (Button) findViewById(R.id.write_memory);
        btnWriteMemory.setOnClickListener(this);
        btnWriteMemory.setOnTouchListener(this);
        btnTagAccess = (Button) findViewById(R.id.tag_access);
        btnTagAccess.setOnClickListener(this);
        btnTagAccess.setOnTouchListener(this);
        btnOption = (Button) findViewById(R.id.option);
        btnOption.setOnClickListener(this);
        btnOption.setOnTouchListener(this);
        btUpdateFirmware = (Button) findViewById(R.id.updatefirmware);
        btUpdateFirmware.setOnClickListener(this);
        btUpdateFirmware.setOnTouchListener(this);
        imgLogo = (ImageView) findViewById(R.id.app_logo);
        imgLogo.setOnClickListener(this);

        setButtonTextColor(COLOR_DISABLED);

        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. initWidgets()");
    }

    // Display Application Verserion
    private void displayVersion() {
        String versionName = "";
        String packageName = getPackageName();

        // Get Application Version
        try {
            versionName = getPackageManager().getPackageInfo(packageName, PackageManager.GET_META_DATA).versionName;
        } catch (NameNotFoundException e) {
            versionName = "";
        }
        txtDemoVersion.setText(versionName + " " + "6");

        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. displayVersion() - [%s]", versionName);
    }

    // Display AsReaderP3xUVerserion
    private void displaySDKVerserion() {
        String version = "";
        try {
            version = AsReaderP3xUManager.getVersion();
        } catch (Exception e) {
            version = "";
        }
        txtSDKVersion.setText(version);

        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. displaySDKVerserion() - [%s]", version);
    }

    // Setup Always Wakeup
    @SuppressLint("InvalidWakeLockTag")
    @SuppressWarnings("deprecation")
    private void setupAlwaysWakeup() {
        if (wakeLock != null)
            return;

        PowerManager powerManager = (PowerManager) this.getSystemService(Activity.POWER_SERVICE);
        if (powerManager != null) {
            wakeLock = powerManager.newWakeLock(PowerManager.FULL_WAKE_LOCK, APP_NAME);
            wakeLock.acquire(1000);
        }

        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. setupAlwayWakeup()");
    }

    // Set Default Wakeup
    private void defaultWakeup() {
        if (wakeLock == null)
            return;

        if (wakeLock.isHeld())
            wakeLock.release();

        wakeLock = null;

        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. defaultWakeup()");
    }

    // Initialize RFID P3xU
    private void initAsReaderP3xU() {
        mAsReaderP3xU = AsReaderP3xUManager.getInstance();
        if (mAsReaderP3xU == null) {
            Toast.makeText(this, "getInstance failed", Toast.LENGTH_SHORT).show();
            return;
        }

        mAsReaderP3xU.setEventListener(this);
        mHandler = new Handler();
        mHandler.post(mConnectDevice);
    }

    private Runnable mConnectDevice = new Runnable() {
        @Override
        public void run() {
            mDevice = mAsReaderP3xU.getCurrentDevice();
            if (mDevice == null) {
                mDevice = usbCdc;
                mAsReaderP3xU.connectDevice(mDevice);
            } else {
                onStateChanged(mAsReaderP3xU, AsReaderP3xUConnectionState.Connected);
            }
        }
    };

    // Enable/Disable All Menu Button
    private void enableMenuButtons(boolean enabled) {
        btnInventory.setEnabled(enabled);
        btnInventory.setTextColor(enabled ? COLOR_ENABLED : COLOR_DISABLED);
        btnReadMemory.setEnabled(enabled);
        btnReadMemory.setTextColor(enabled ? COLOR_ENABLED : COLOR_DISABLED);
        btnWriteMemory.setEnabled(enabled);
        btnWriteMemory.setTextColor(enabled ? COLOR_ENABLED : COLOR_DISABLED);
        btnTagAccess.setEnabled(enabled);
        btnTagAccess.setTextColor(enabled ? COLOR_ENABLED : COLOR_DISABLED);
        btnOption.setEnabled(enabled);
        btnOption.setTextColor(enabled ? COLOR_ENABLED : COLOR_DISABLED);
        btUpdateFirmware.setEnabled(enabled);
        btUpdateFirmware.setTextColor(enabled ? COLOR_ENABLED : COLOR_DISABLED);
        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. enableMenuButtons(%s)", enabled);
    }

    // Load Configuration
    private void loadConfig() {
        SharedPreferences prefs = getSharedPreferences(APP_NAME, MODE_PRIVATE);
    }

    private void setButtonTextColor(int color) {
        btnInventory.setTextColor(color);
        btnReadMemory.setTextColor(color);
        btnWriteMemory.setTextColor(color);
        btnTagAccess.setTextColor(color);
        btnOption.setTextColor(color);
        btUpdateFirmware.setTextColor(color);
    }

    private void setDisconnectedState() {

        txtFirmwareVersion.setText("");
        txtHardwareVersion.setText("");
        txtAddress.setText("");
        txtSerialNumber.setText("");
        txtGlobalBand.setText("");

        enableMenuButtons(false);
        imgLogo.setImageResource(R.drawable.asreader_log_off);
        WaitDialog.hide();
        setProgressBarIndeterminateVisibility(false);

        AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. setDisconnectedState()");
    }

    private String getRequestCode(int requestCode) {
        switch (requestCode) {
            case VIEW_INVENTORY:
                return "VIEW_INVENTORY";
            case AccessActivity.VIEW_READ_MEMORY:
                return "VIEW_READ_MEMORY";
            case AccessActivity.VIEW_WRITE_MEMORY:
                return "VIEW_WRITE_MEMORY";
            case AccessActivity.VIEW_TAG_ACCESS:
                return "VIEW_TAG_ACCESS";
            case VIEW_OPTION:
                return "VIEW_OPTION";
        }
        return "";
    }

    private String getResultCode(int resultCode) {
        switch (resultCode) {
            case Activity.RESULT_CANCELED:
                return "RESULT_CANCELED";
            case Activity.RESULT_OK:
                return "RESULT_OK";
            case Activity.RESULT_FIRST_USER:
                return "RESULT_FIRST_USER";
            case RESULT_DISCONNECTED:
                return "RESULT_DISCONNECTED";
        }
        return "";
    }

    private String getOnClickId(int id) {
        switch (id) {
            case R.id.inventory:
                return "inventory";
            case R.id.read_memory:
                return "read_memory";
            case R.id.write_memory:
                return "write_memory";
            case R.id.tag_access:
                return "tag_access";
            case R.id.option:
                return "option";
        }
        return "";
    }

    @SuppressLint({"NewApi", "ClickableViewAccessibility"})
    @Override
    public boolean onTouch(View v, MotionEvent event) {

        int id = v.getId();
        int action = event.getAction();
        switch (id) {
            case R.id.inventory:
                switch (action) {
                    case MotionEvent.ACTION_DOWN:
                        btnInventory.setBackgroundColor(Color.LTGRAY);
                        break;
                    case MotionEvent.ACTION_UP:
                    case MotionEvent.ACTION_CANCEL:
                        btnInventory.setBackgroundColor(Color.TRANSPARENT);
                        break;
                }
                break;
            case R.id.read_memory:
                switch (action) {
                    case MotionEvent.ACTION_DOWN:
                        btnReadMemory.setBackgroundColor(Color.LTGRAY);
                        break;
                    case MotionEvent.ACTION_UP:
                    case MotionEvent.ACTION_CANCEL:
                        btnReadMemory.setBackgroundColor(Color.TRANSPARENT);
                        break;
                }
                break;
            case R.id.write_memory:
                switch (action) {
                    case MotionEvent.ACTION_DOWN:
                        btnWriteMemory.setBackgroundColor(Color.LTGRAY);
                        break;
                    case MotionEvent.ACTION_UP:
                    case MotionEvent.ACTION_CANCEL:
                        btnWriteMemory.setBackgroundColor(Color.TRANSPARENT);
                        break;
                }
                break;
            case R.id.tag_access:
                switch (action) {
                    case MotionEvent.ACTION_DOWN:
                        btnTagAccess.setBackgroundColor(Color.LTGRAY);
                        break;
                    case MotionEvent.ACTION_UP:
                    case MotionEvent.ACTION_CANCEL:
                        btnTagAccess.setBackgroundColor(Color.TRANSPARENT);
                        break;
                }
                break;
            case R.id.option:
                switch (action) {
                    case MotionEvent.ACTION_DOWN:
                        btnOption.setBackgroundColor(Color.LTGRAY);
                        break;
                    case MotionEvent.ACTION_UP:
                    case MotionEvent.ACTION_CANCEL:
                        btnOption.setBackgroundColor(Color.TRANSPARENT);
                        break;
                }
                break;
            case R.id.updatefirmware:
                switch (action) {
                    case MotionEvent.ACTION_DOWN:
                        btUpdateFirmware.setBackgroundColor(Color.LTGRAY);
                        break;
                    case MotionEvent.ACTION_UP:
                    case MotionEvent.ACTION_CANCEL:
                        btUpdateFirmware.setBackgroundColor(Color.TRANSPARENT);
                        break;
                }
                break;
        }

        return false;
    }



    public class InitRunnable implements Runnable {

        private final String _firmwareVersion;
        private final String _hardwareVersion;
        private final String _serialNumber;
        private final String _rfVersion;
        private final String _globalBand;

        public InitRunnable(String firmwareVersion, String hardwareVersion, String serialNumber, String rfVersion, String globalBand) {
            _firmwareVersion = firmwareVersion;
            _hardwareVersion = hardwareVersion;
            _serialNumber = serialNumber;
            _rfVersion = rfVersion;
            _globalBand = globalBand;
        }

        @Override
        public void run() {
            txtFirmwareVersion.setText(_firmwareVersion);
            txtHardwareVersion.setText(_hardwareVersion);
            txtSerialNumber.setText(_serialNumber);
            txtAddress.setText(_rfVersion);
            txtGlobalBand.setText(_globalBand);
            firmwareVersionStr = _firmwareVersion;
            String[] hardVersionString = _hardwareVersion.split("\\.");
            if (hardVersionString.length == 3) {
                String str = hardVersionString[2];
                if (str.length() > 0) {
                    mUserConfigs.setHardwareVersion(str);
                } else {
                    mUserConfigs.setHardwareVersion("");
                }
            } else {
                mUserConfigs.setHardwareVersion("");
            }
            mUserConfigs.Save();
            WaitDialog.hide();
            imgLogo.setImageResource(R.drawable.asreader_log_on);
            enableMenuButtons(true);

        }
    }
}
