package com.asreader.p3xu.view;

import java.util.ArrayList;
import java.util.Locale;

import com.asreader.p3xu.AsReaderP3xU;
import com.asreader.p3xu.device.type.AsReaderP3xUConnectionState;
import com.asreader.p3xu.rfid.exception.AsReaderP3xUException;
import com.asreader.p3xu.rfid.params.AsReaderP3xUPowerRange;
import com.asreader.p3xu.rfid.type.AsReaderP3xUActionState;
import com.asreader.p3xu.rfid.type.AsReaderP3xUQuerySession;
import com.asreader.p3xu.rfid.type.AsReaderP3xUSessionFlag;
import com.asreader.p3xu.rfid.type.AsReaderP3xUResultCode;
import com.asreader.p3xu.device.type.AsReaderP3xUExceptionCode;
import com.asreader.p3xu.device.type.AsReaderP3xUKeyState;
import com.asreader.p3xu.device.type.AsReaderP3xUKeyType;
import com.asreader.R;
import com.asreader.p3xu.diagnostics.AsReaderP3xUASRLog;
import com.asreader.support.andro.app.rfid.adapter.PowerRangeAdapter;
import com.asreader.support.andro.app.rfid.adapter.SpinnerAdapter;
import com.asreader.support.andro.app.rfid.data.SaveAsCSV;
import com.asreader.support.andro.app.rfid.dialog.WaitDialog;
import com.asreader.support.andro.app.rfid.filter.InputFilterMinMax;
import com.asreader.support.andro.app.rfid.util.ResUtil;
import com.asreader.support.andro.app.rfid.util.UserConfigs;
import com.asreader.support.andro.app.rfid.view.base.BaseInventoryActivity;

import android.app.ActionBar;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.text.InputFilter;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.WindowManager;
import android.view.inputmethod.InputMethodManager;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

public class InventoryActivity extends BaseInventoryActivity {

	private static final String TAG = InventoryActivity.class.getSimpleName();

	private TextView txtTotalCount;
	private TextView txtSpeed;
	private LinearLayout layoutOption;
	private TextView txtPowerGain;
	private TextView txtDisplayPC;
	private TextView txtContinuousMode;
	private TextView txtReportRssi;
	private TextView txtOperationTime;
	private TextView txtQuerySession;
	private TextView txtQueryTarget;

	private AsReaderP3xUPowerRange mPowerRange;
	private int mCurrentPowerGain;
	private boolean mDisplayPC;
	private boolean mContinuousMode;
	private boolean mReportRssi;
	private int mOperationTime;
	private AsReaderP3xUQuerySession mQuerySession;
	private AsReaderP3xUSessionFlag mQueryTarget;

	private long mLastTime;
	private double mTagSpeed;

	private long mSpeedTagCount;

	private int mRestartTime;
	private boolean mIsRunRestart;
	private Handler mHandler;
	private UserConfigs mUserConfigs;
	private ArrayList<String> tagArray;
	private long mTotalTagCounts;
	private boolean isRunning;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_inventory);
		mUserConfigs = UserConfigs.getInstance(this.getApplicationContext());
//		ActionBar actionBar = getActionBar();
//		if(actionBar != null) {
//			actionBar.setDisplayOptions(ActionBar.DISPLAY_SHOW_CUSTOM);
//			actionBar.setCustomView(R.layout.activity_title);
//			actionBar.setDisplayHomeAsUpEnabled(true);
//			// actionBar.setDisplayShowHomeEnabled(true);
//			actionBar.setHomeAsUpIndicator(R.mipmap.icon_left_return); // zsj 2021-03-05 ActionBar 设置左边返回按钮的图片
//			actionBar.setDisplayShowHomeEnabled(false); // zsj 2021-02-25 隐藏ActionBar左边icon
//		}

		ActionBar actionBar = getActionBar();
		if (actionBar != null) {
			actionBar.setDisplayOptions(ActionBar.DISPLAY_SHOW_CUSTOM);
			actionBar.setCustomView(R.layout.activity_title);
			actionBar.setDisplayHomeAsUpEnabled(true);
			actionBar.setDisplayShowHomeEnabled(true);
		}

		TextView titleText = (TextView)findViewById(R.id.title_text);
		titleText.setText(R.string.inventory_name);
		final int version = Build.VERSION.SDK_INT;
		if(version >= Build.VERSION_CODES.M)
			titleText.setTextColor(getColor(getApplicationContext(), R.color.black));

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onCreate()");

		mHandler = new Handler();

		// Create Reader
		createAsReaderP3xU();

		// Initialize Mask
		initMask();

		// Initialize Widget
		initWidgets();

		// Initialize Reader
		initAsReaderP3xU();

		mTotalTagCounts = 0;
		mLastTime = 0;
		mTagSpeed = 0.0;
		mRestartTime = 0;
		mIsRunRestart = false;

		// Enable action widget
		enableActionWidgets(true);

		mSpeedTagCount = 0;
		mCurrentPowerGain = 0;
		mDisplayPC = true;
		mContinuousMode = false;
		mReportRssi = false;
		mOperationTime = 0;
		mQuerySession = AsReaderP3xUQuerySession.S0;
		mQueryTarget = AsReaderP3xUSessionFlag.A;
		tagArray = new ArrayList();

		this.getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
	}
	@Override
	protected void onResume() {
		super.onResume();
		if(getAsReaderP3xU().getState() == AsReaderP3xUConnectionState.Connected) {
			// Load Option
			WaitDialog.show(this, R.string.load_option);
			new Thread(mLoadOption).start();
		}
	}
	@Override
	protected void onDestroy() {

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onDestroy()");
		this.getWindow().clearFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
		// Destroy Reader
		destroyAsReaderP3xU();

		adpTags.shutDown();

		super.onDestroy();
	}

	@Override
	public void onBackPressed() {
		if(layoutOption.getVisibility() == View.VISIBLE) {
			layoutOption.setVisibility(View.GONE);
			return;
		}
		super.onBackPressed();
	}

	@Override
	protected void onStart() {
		super.onStart();

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onStart()");
	}

	@Override
	protected void onStop() {
		if(getAsReaderP3xU().getAction() == AsReaderP3xUActionState.Inventory)
			getAsReaderP3xU().stop();
		isRunning = false;
		super.onStop();
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onCreateOptionsMenu()");
		getMenuInflater().inflate(R.menu.save_file, menu);
		return true;
	}

	@Override
	public boolean onPrepareOptionsMenu(Menu menu) {
		MenuItem mnuSaveFile;

		mnuSaveFile = menu.findItem(R.id.save_csv);
		mnuSaveFile.setEnabled(getAsReaderP3xU().getAction() == AsReaderP3xUActionState.Stop);

		return super.onPrepareOptionsMenu(menu);
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		int id = item.getItemId();

		switch (id) {
		case R.id.save_csv:
			if(adpTags.getCount() < 1)
				return true;

			SaveAsCSV task = new SaveAsCSV(InventoryActivity.this, SaveAsCSV.DATA_TYPE_RFID, adpTags, mDisplayPC);
			task.execute();
			return true;
		}
		return super.onOptionsItemSelected(item);
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
	}

	private void displayPowerGain() {
		txtPowerGain.setText(String.format(Locale.US, "%.1f dBm", (double) mCurrentPowerGain / 10.0));
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. displayPowerGain(%d)", mCurrentPowerGain);
	}

	private void displayDisplayPC() {
		txtDisplayPC.setText(mDisplayPC ? "On" : "Off");
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. displayDisplayPC(%s)", mDisplayPC);
	}

	private void displayContinuousMode() {
		txtContinuousMode.setText(mContinuousMode ? "On" : "Off");
	}

	private void displayReportRSSI() {
		txtReportRssi.setText(mReportRssi ? "On" : "Off");
	}

	private void displayOperationTime() {
		// txtOperationTime.setText(String.format(Locale.US, "%d ms", mOperationTime));  // zsj-2020-12-28 注释 - 毫秒 改 秒
		txtOperationTime.setText(String.format(Locale.US, "%d s", mOperationTime));
	}

	private void displayQuerySession() {
		txtQuerySession.setText(mQuerySession.toString());
	}

	private void displayQueryTarget() {
		txtQueryTarget.setText(mQueryTarget.toString());
	}

	private Runnable mLoadOption = new Runnable() {

		@Override
		public void run() {
			try {
				mPowerRange = getAsReaderP3xU().getPowerGainRange();
			} catch (Exception e) {
//				mPowerRange = new PowerRange(10, 260);
			}

			try {
				mCurrentPowerGain = getAsReaderP3xU().getPowerGain();
			} catch (AsReaderP3xUException e) {
				mCurrentPowerGain = 0;
				AsReaderP3xUASRLog.e(TAG, e, "ERROR. initReader() - Failed to get power gain");
			}

			try {
				mContinuousMode = mUserConfigs.getContinuousMode();
				getAsReaderP3xU().setContinuousMode(mContinuousMode);
			} catch (AsReaderP3xUException e) {
				AsReaderP3xUASRLog.e(TAG, e, "ERROR. initReader() - Failed to set continuous mode");
			}

			try {
				mReportRssi = getAsReaderP3xU().getReportRSSI();
			} catch (AsReaderP3xUException e) {
				AsReaderP3xUASRLog.e(TAG, e, "ERROR. initReader() - Failed to get report rssi");
			}
			try {
				mOperationTime = getAsReaderP3xU().getOperationTime();
			} catch (AsReaderP3xUException e) {
				mOperationTime = 0;
				AsReaderP3xUASRLog.e(TAG, e, "ERROR. initReader() - Failed to get operation time");
			}
			try {
				mQuerySession = getAsReaderP3xU().getQuerySession();
			} catch (AsReaderP3xUException e) {
				AsReaderP3xUASRLog.e(TAG, e, "ERROR. initReader() - Failed to get query session");
			}
			try {
				mQueryTarget = getAsReaderP3xU().getSessionFlag();
			} catch (AsReaderP3xUException e) {
				AsReaderP3xUASRLog.e(TAG, e, "ERROR. initReader() - Failed to get query target");
			}

			mDisplayPC = mUserConfigs.getDisplayPc();

			mHandler.post(new Runnable() {

				@Override
				public void run() {
					setDisplayPC(mDisplayPC);
					displayPowerGain();
					displayDisplayPC();
					displayContinuousMode();
					displayReportRSSI();
					displayOperationTime();
					displayQuerySession();
					displayQueryTarget();
					WaitDialog.hide();
					enableActionWidgets(true);
				}

			});
		}

	};

	private void showPowerGainDialog() {

		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView power = (ListView) root.findViewById(R.id.list);
		final PowerRangeAdapter adapter = new PowerRangeAdapter(this, mPowerRange,
				android.R.layout.simple_list_item_single_choice);
		power.setAdapter(adapter);
		power.setChoiceMode(ListView.CHOICE_MODE_SINGLE);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.power_gain);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				int position = power.getCheckedItemPosition();
				int power = adapter.getItemValue(position);

				try {
					getAsReaderP3xU().setPowerGain(power); // test：设置 25，  write:7e77703235300d0a 回（~wp250） 7e5770303030300d0a0a303030303030464b303031310d0a （~Wp0000）
					mCurrentPowerGain = power;
					mUserConfigs.setPowerGain(power);
					mUserConfigs.Save();
				} catch (AsReaderP3xUException e) {
					AsReaderP3xUASRLog.e(TAG, e, "ERROR. $showPowerGainDialog() - Failed to set power gain (%d)", power);
				}

				displayPowerGain();
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showPowerGinaDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showPowerGinaDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showPowerGinaDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getItemPosition(mCurrentPowerGain);
				power.setItemChecked(position, true);
				power.setSelectionFromTop(position, 0);
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showPowerGinaDialog().onShow()");
			}
		});
		dialog.show();

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showPowerGinaDialog()");
	}

	private void showDisplayPcDialog() {
		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
		adapter.addItem(0, "Off");
		adapter.addItem(1, "On");
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.display_pc);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					int position = value.getCheckedItemPosition();
					mDisplayPC = adapter.getValue(position) > 0;
					setDisplayPC(mDisplayPC);
					mUserConfigs.setDisplayPc(mDisplayPC);
					mUserConfigs.Save();
				} catch (Exception e) {
					AsReaderP3xUASRLog.e(TAG, "ERROR.  showDisplayPcDialog().$PositiveButton.onClick() - [%s]"
								, e.getMessage());
				}
				displayDisplayPC();
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showDisplayPcDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showDisplayPcDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showDisplayPcDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mDisplayPC ? 1 : 0);
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showDisplayPcDialog().onShow()");
			}
		});
		dialog.show();
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showDisplayPcDialog()");
	}

	private void showContinuousModeDialog() {
		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
		adapter.addItem(0, "Off");
		adapter.addItem(1, "On");
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.continuous_mode);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					int position = value.getCheckedItemPosition();
					mContinuousMode = adapter.getValue(position) > 0;
					getAsReaderP3xU().setContinuousMode(mContinuousMode);
					mUserConfigs.setContinuousMode(mContinuousMode);
					mUserConfigs.Save();
				} catch (Exception e) {
					AsReaderP3xUASRLog.e(TAG, "ERROR.  showContinuousModeDialog().$PositiveButton.onClick() - [%s]"
							, e.getMessage());
				}
				displayContinuousMode();
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showContinuousModeDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showContinuousModeDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showContinuousModeDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mContinuousMode ? 1 : 0);
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showContinuousModeDialog().onShow()");
			}
		});
		dialog.show();
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showContinuousModeDialog()");
	}

	private void showReportRssiDialog() {
		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
		adapter.addItem(0, "Off");
		adapter.addItem(1, "On");
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.report_rssi);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					int position = value.getCheckedItemPosition();
					mReportRssi = adapter.getValue(position) > 0;
					getAsReaderP3xU().setReportRSSI(mReportRssi);
					mUserConfigs.setReportRssi(mReportRssi);
					mUserConfigs.Save();
					setReportRssiMode(mReportRssi);
					mTotalTagCounts = 0;
					tagArray = new ArrayList();
				} catch (Exception e) {
					AsReaderP3xUASRLog.e(TAG, "ERROR.  showReportRssiDialog().$PositiveButton.onClick() - [%s]"
							, e.getMessage());
				}
				displayReportRSSI();
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showReportRssiDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				mTotalTagCounts = 0;
				tagArray = new ArrayList();
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showReportRssiDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				mTotalTagCounts = 0;
				tagArray = new ArrayList();
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showReportRssiDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mReportRssi ? 1 : 0);
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showReportRssiDialog().onShow()");
			}
		});
		dialog.show();
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showReportRssiDialog()");
	}

	private void showOperationTimeDialog() {

		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_input_unit, null);
		final TextView unit = (TextView) root.findViewById(R.id.unit);
		unit.setText(R.string.unit_sec);

		final EditText operationTime = (EditText) root.findViewById(R.id.value);

		InputFilter[] filters = new InputFilter[] { new InputFilterMinMax(0, 1800) };
		operationTime.setFilters(filters);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.operation_time);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {

				try {
					mOperationTime = Integer.parseInt(operationTime.getText().toString());
				} catch (Exception e) {
					mOperationTime = 0;
				}
				try {
					getAsReaderP3xU().setOperationTime(mOperationTime);
					mUserConfigs.setOperationTime(mOperationTime);
					mUserConfigs.Save();
				} catch (Exception e) {
					AsReaderP3xUASRLog.e(TAG, "ERROR.  showOperationTimeDialog().$PositiveButton.onClick() - [%s]"
							, e.getMessage());
				}
				displayOperationTime();
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null)
					imm.hideSoftInputFromWindow(operationTime.getWindowToken(), 0);
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showOperationTimeDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null)
					imm.hideSoftInputFromWindow(operationTime.getWindowToken(), 0);
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showOperationTimeDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null)
					imm.hideSoftInputFromWindow(operationTime.getWindowToken(), 0);
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showOperationTimeDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				operationTime.setText(String.format(Locale.US, "%d", mOperationTime));
				operationTime.selectAll();
				operationTime.requestFocus();
				InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
				if(imm != null)
					imm.showSoftInput(operationTime, InputMethodManager.SHOW_FORCED);
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showOperationTimeDialog().onShow()");
			}
		});
		dialog.show();

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showOperationTimeDialog()");
	}

	private void showQuerySessionDialog() {
		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
		for (AsReaderP3xUQuerySession item : AsReaderP3xUQuerySession.values()) {
			adapter.addItem(item.getValue(), item.toString());
		}
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.session_inventory);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					int position = value.getCheckedItemPosition();
					mQuerySession = AsReaderP3xUQuerySession.valueOf(adapter.getValue(position));
					getAsReaderP3xU().setQuerySession(mQuerySession);
					mUserConfigs.setSession(mQuerySession);
					mUserConfigs.Save();
				} catch (Exception e) {
					AsReaderP3xUASRLog.e(TAG, "ERROR.  showQuerySessionDialog().$PositiveButton.onClick() - [%s]"
							, e.getMessage());
				}
				displayQuerySession();
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showQuerySessionDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showQuerySessionDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showQuerySessionDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mQuerySession.getValue());
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showQuerySessionDialog().onShow()");
			}
		});
		dialog.show();
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showQuerySessionDialog()");
	}

	private void showQueryTargetDialog() {
		if (!mIsEnabled)
			return;
		String deviceModel = "P3xU";
		try {
			Thread.sleep(100);
			String devieModel = getAsReaderP3xU().getDeviceModel();
			if (devieModel.equals("0")) {
				deviceModel = "P3xU";
			} else if (devieModel.equals("1")) {
				//D3
				deviceModel = "D3";
			}
		} catch (Exception e) {
			AsReaderP3xUASRLog.e(TAG, "ERROR. showQValueDialog().getDeviceModel", e.getMessage());
		}
		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);
		for (AsReaderP3xUSessionFlag item : AsReaderP3xUSessionFlag.values()) {
			if (deviceModel.equals("D3")) {
				if (item.getValue() == 2) {
					break;
				}
			}
			adapter.addItem(item.getValue(), item.toString());
		}
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.sesstion_flag_target);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				try {
					int position = value.getCheckedItemPosition();
					mQueryTarget = AsReaderP3xUSessionFlag.valueOf(adapter.getValue(position));
					getAsReaderP3xU().setSessionFlag(mQueryTarget);
					mUserConfigs.setTarget(mQueryTarget);
					mUserConfigs.Save();
				} catch (Exception e) {
					AsReaderP3xUASRLog.e(TAG, "ERROR.  showQueryTargetDialog().$PositiveButton.onClick() - [%s]"
							, e.getMessage());
				}
				displayQueryTarget();
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showQueryTargetDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showQueryTargetDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showQueryTargetDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mQueryTarget.getValue());
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showQueryTargetDialog().onShow()");
			}
		});
		dialog.show();
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showQueryTargetDialog()");
	}

	private void showErrorDialog(int code) {

		final AsReaderP3xUExceptionCode readerException = AsReaderP3xUExceptionCode.valueOf(code);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.exception_occurred);
		builder.setMessage(String.format(Locale.US, "Value : 0x%X\n%s", readerException.getValue(), readerException.toString()));
		builder.setCancelable(false);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {
			@Override
			public void onClick(DialogInterface dialog, int which) {

				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showErrorDialog().$PositiveButton.onClick()");
			}
		});

		AlertDialog dialog = builder.create();
		dialog.show();
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showAlgorithmDialog()");
	}

	@Override
	public void onClick(View v) {
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onClick(%s)", ResUtil.getId(v.getId()));
		switch (v.getId()) {
		case R.id.option:
			if(layoutOption.getVisibility() == View.VISIBLE) {
				layoutOption.setVisibility(View.GONE);
			} else {
				layoutOption.setVisibility(View.VISIBLE);
			}
			break;
		case R.id.inventory_power_gain:
			showPowerGainDialog();
			break;
		case R.id.inventory_display_pc:
			showDisplayPcDialog();
			break;
		case R.id.inventory_continuous_mode:
			showContinuousModeDialog();
			break;
		case R.id.inventory_report_rssi:
			showReportRssiDialog();
			break;
		case R.id.inventory_operation_time:
			showOperationTimeDialog();
			break;
		case R.id.inventory_query_session:
			showQuerySessionDialog();
			break;
		case R.id.inventory_query_target:
			showQueryTargetDialog();
			break;
		case R.id.mask:
			asReaderP3xUSelectMaskEpcParam.setMask("");
			break;
		}

		super.onClick(v);
	}

	@Override
	public void onReadTag(AsReaderP3xU asReaderP3xU, AsReaderP3xUActionState action, String tag, float rssi) {
//		long time = System.currentTimeMillis();
//		double interval;

		AsReaderP3xUASRLog.d(TAG, AsReaderP3xUASRLog.DEBUG, "EVENT. onReadTag(%s, [%s], %.2f)", action, tag, rssi);

		mTotalTagCounts++;
		mSpeedTagCount++;
//		if (mLastTime == 0) {
//			mTagSpeed = 0.0;
//			mLastTime = time;
//		} else {
//			interval = (double) (time - mLastTime) / 1000.0;
//			mTagSpeed = (double) mSpeedTagCount / interval;
//		}

		super.onReadTag(asReaderP3xU, action, tag, rssi);
		txtTotalCount.setText(String.format(Locale.US, "%d", mTotalTagCounts));
//		txtSpeed.setText(String.format(Locale.US, "%.2f tps", mTagSpeed));
	}

	@Override
	public void onActionChanged(AsReaderP3xU asReaderP3xU, AsReaderP3xUActionState action) {

		if (mIsRunRestart) {
			adpTags.notifyDataSetChanged();
			if (action != AsReaderP3xUActionState.Stop) {
				enableActionWidgets(true);
			}
		} else {
			super.onActionChanged(asReaderP3xU, action);
		}

		if(action == AsReaderP3xUActionState.Stop){
			mSpeedTagCount = 0;
			mLastTime =0;
			if (mRestartTime > 0 && mIsRunRestart) {
				mHandler.postDelayed(mRestartHandler, mRestartTime);
			}
		}
	}

	@Override
	public void onKeyEvent(AsReaderP3xU asReaderP3xU, AsReaderP3xUKeyType key, AsReaderP3xUKeyState state) {
		if (key == AsReaderP3xUKeyType.Trigger) {
			if (state == AsReaderP3xUKeyState.KeyUp) {
				if (getAsReaderP3xU().getAction() != AsReaderP3xUActionState.Stop) {
					stopInventory();
				} else {
					startInventory();
				}
			}
		}
	}

	// Initialize Reader
	@Override
	protected void initAsReaderP3xU() {
		super.initAsReaderP3xU();

		try {
			adpTags.setReportRSSI(getAsReaderP3xU().getReportRSSI());
			getDB();
		} catch (AsReaderP3xUException e) {
			AsReaderP3xUASRLog.e(TAG, e, "ERROR. initReader() - Failed to get report rssi mode");
		}

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. initReader()");
	}

	// Initialize Widgets
	@Override
	protected void initWidgets() {
		super.initWidgets();

		txtTotalCount = (TextView) findViewById(R.id.total_tag_count);
		txtTotalCount.setText(String.format(Locale.US, "%d", mTotalTagCounts));
		txtSpeed = (TextView) findViewById(R.id.speed);
		txtSpeed.setText(R.string.init_tps);
		layoutOption = (LinearLayout) findViewById(R.id.inventory_option);
		// layoutOption.setVisibility(View.GONE); // zsj 2021-02-25 默认显示Power Gain等设置项，点击[Option]后，隐藏这些设置项（再点再显示）
		layoutOption.setBackgroundColor(getColor(getApplicationContext(), R.color.message_background));
		txtPowerGain = (TextView) findViewById(R.id.inventory_power_gain);
		txtPowerGain.setOnClickListener(this);
		txtDisplayPC = (TextView) findViewById(R.id.inventory_display_pc);
		txtDisplayPC.setOnClickListener(this);
		txtContinuousMode = (TextView) findViewById(R.id.inventory_continuous_mode);
		txtContinuousMode.setOnClickListener(this);
		txtReportRssi = (TextView) findViewById(R.id.inventory_report_rssi);
		txtReportRssi.setOnClickListener(this);
		txtOperationTime = (TextView) findViewById(R.id.inventory_operation_time);
		txtOperationTime.setOnClickListener(this);
		txtQuerySession = (TextView) findViewById(R.id.inventory_query_session);
		txtQuerySession.setOnClickListener(this);
		txtQueryTarget = (TextView) findViewById(R.id.inventory_query_target);
		txtQueryTarget.setOnClickListener(this);

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. initWidgets()");
	}

	// Enable/Disable Action Widgets
	@Override
	protected void enableActionWidgets(boolean enabled) {
		super.enableActionWidgets(enabled);
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. enableActionWidgets(%s)", enabled);
	}

	@Override
	protected void clearWidgets() {
		super.clearWidgets();

//		mSpeedTagCount =0;
		mLastTime = 0;
		mTagSpeed = 0.0;
		mTotalTagCounts = 0;
		tagArray = new ArrayList();
		txtTotalCount.setText(String.format(Locale.US, "%d", mTotalTagCounts));
		txtSpeed.setText(String.format(Locale.US, "%.2f tps", mTagSpeed));

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. clearWidgets()");
	}

	@Override
	protected void startInventory() {
//		if(getAsReaderP3xU().getAction() != AsReaderP3xUActionState.Stop)
//			return;

		mIsRunRestart = mRestartTime > 0;
		tagArray.clear();
		tagArray.add(0, String.valueOf(mTotalTagCounts));
		isRunning = true;
		new Thread(new Runnable() {
			public void run() {
				while (isRunning) {
					tagArray.add(0, String.valueOf(mTotalTagCounts));

					if (tagArray.size() > 2) {
						tagArray.remove(tagArray.size() - 1);
					}

					String last = tagArray.get(0);
					String first = tagArray.get(tagArray.size() - 1);
					mTagSpeed = (Long.valueOf(last) - Long.valueOf(first)) ;
					Log.e("Test~~~mTagSpeed", String.valueOf(mTagSpeed));
					Log.e("Test~~~isRunning", String.valueOf(isRunning));
					runOnUiThread(calculateTheMean);
					try {
						Thread.sleep(1000);
					} catch (InterruptedException e) {
						throw new RuntimeException(e);
					}
				}
			}

		}).start();
		super.startInventory();
	}

	@Override
	protected void stopInventory() {
//		if(getAsReaderP3xU().getAction() == AsReaderP3xUActionState.Stop)
//			return;
		isRunning = false;
		mIsRunRestart = false;
		super.stopInventory();
	}

	private Runnable calculateTheMean = new Runnable() {
		@Override
		public void run() {
			txtSpeed.setText(String.format(Locale.US, "%.2f tps", mTagSpeed));
		}
	};

	private Runnable mRestartHandler = new Runnable() {

		@Override
		public void run() {
			if (getAsReaderP3xU().inventory() != AsReaderP3xUResultCode.NoError) {
				AsReaderP3xUASRLog.e(TAG, "ERROR. $mRestartHandler.run() - Failed to start inventory()");
				enableActionWidgets(true);
				return;
			}
			AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. $mRestartHandler.run()");
		}
	};
}
