package com.asreader.p3xu.view;

import java.lang.ref.WeakReference;
import java.util.Locale;

import com.asreader.p3xu.AsReaderP3xU;
import com.asreader.p3xu.rfid.exception.AsReaderP3xUException;
import com.asreader.p3xu.rfid.type.AsReaderP3xUActionState;
import com.asreader.p3xu.rfid.type.AsReaderP3xUMemoryBank;
import com.asreader.p3xu.rfid.type.AsReaderP3xUResultCode;
import com.asreader.R;
import com.asreader.p3xu.device.type.AsReaderP3xUKeyState;
import com.asreader.p3xu.device.type.AsReaderP3xUKeyType;
import com.asreader.p3xu.diagnostics.AsReaderP3xUASRLog;
import com.asreader.support.andro.app.rfid.adapter.MemoryListAdapter;
import com.asreader.support.andro.app.rfid.adapter.SpinnerAdapter;
import com.asreader.support.andro.app.rfid.dialog.WaitDialog;
import com.asreader.support.andro.app.rfid.util.ResUtil;
import com.asreader.support.andro.app.rfid.view.base.MemoryActivity;

import android.app.ActionBar;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.ScrollView;
import android.widget.TextView;
import android.widget.Toast;

public class ReadMemoryActivity extends MemoryActivity {

	private static final String TAG = ReadMemoryActivity.class.getSimpleName();

	private static final int DEFAULT_MEM_LENGTH = 2;

	private ListView lstValue;
	private TextView txtLength;
	private Button btnRead;

	private MemoryListAdapter adpValue;

	private int mLength;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_read_memory);
		ActionBar actionBar = getActionBar();
		if (actionBar != null) {
			actionBar.setDisplayOptions(ActionBar.DISPLAY_SHOW_CUSTOM);
			actionBar.setCustomView(R.layout.activity_title);
			actionBar.setDisplayHomeAsUpEnabled(true);
			actionBar.setDisplayShowHomeEnabled(true);
		}
		TextView titleText = (TextView)findViewById(R.id.title_text);
		titleText.setText(R.string.read_memory_name);
		final int version = Build.VERSION.SDK_INT;
		if(version >= Build.VERSION_CODES.M)
			titleText.setTextColor(getColor(getApplicationContext(), R.color.black));

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onCreate()");

		// Create Reader
		createAsReaderP3xU();

		// Initialize Widget
		initWidgets();

		// Initialize Reader
		initAsReaderP3xU();

		InitTask task = new InitTask(this);
		task.execute();
		
	}

	@Override
	protected void onDestroy() {

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onDestroy()");

		// Destroy Reader
		destroyAsReaderP3xU();

		super.onDestroy();
	}

	@Override
	protected void onStart() {
		super.onStart();
		
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onStart()");
	}

	@Override
	protected void onStop() {
		
		if(getAsReaderP3xU().getAction() == AsReaderP3xUActionState.ReadMemory)
			getAsReaderP3xU().stop();

		// Rollback Mask
		exitMask();

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onStop()");
		super.onStop();
	}
	
	@Override
	protected void onResume() {
		super.onResume();
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onResume()");
	}

	@Override
	protected void onPause() {
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onPause()");
		super.onPause();
	}

	@Override
	public void onClick(View v) {

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onClick(%s)", ResUtil.getId(v.getId()));

		switch (v.getId()) {
		case R.id.length:
			showMemLengthDialog();
			break;
		case R.id.action_read:
			if (getAsReaderP3xU().getAction() == AsReaderP3xUActionState.Stop) {
				readMemory();
			} else {
				stopAction();
			}
			break;
		}
		super.onClick(v);
	}
	
	@Override
	public void onKeyEvent(AsReaderP3xU asReaderP3xU, AsReaderP3xUKeyType key, AsReaderP3xUKeyState state) {
		if(key == AsReaderP3xUKeyType.Trigger) {
			if(state == AsReaderP3xUKeyState.KeyDown) {
				if(getAsReaderP3xU().getAction() != AsReaderP3xUActionState.Stop)
					return;
				readMemory();
			} else {
				if(getAsReaderP3xU().getAction() == AsReaderP3xUActionState.Stop)
					return;
				stopAction();
			}
		}
	}

	@Override
	public void onActionChanged(AsReaderP3xU asReaderP3xU, AsReaderP3xUActionState action) {

		AsReaderP3xUASRLog.d(TAG, AsReaderP3xUASRLog.DEBUG, "EVENT. onActionChanged(%s)", action);

		enableActionWidgets(true);

		if (action == AsReaderP3xUActionState.Stop) {
			if (!isActionResult()) {
				outputMessage(R.string.read_memory_intro_message);
			}
		}
	}

	@Override
	public void onAccessResult(AsReaderP3xU asReaderP3xU, AsReaderP3xUResultCode code, AsReaderP3xUActionState action, String epc, String data) {
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "EVENT. onAccessResult(%s, %s, [%s], [%s])", code, action, epc == null ? "" : epc,
				data == null ? "" : data);

		setActionResult();

		if (code != AsReaderP3xUResultCode.NoError) {
			outputFailMessage(code.toString());
			return;
		}

		if(epc != null)
			outputSelection(epc.length() > 4 ? epc.substring(4) : epc);

		outputSuccessMessage(R.string.access_success);
		int offset = getMemOffset();
		adpValue.setValue(offset, data);
	}

	// Read Memroy
	private void readMemory() {
		
		clearWidgets();
		enableActionWidgets(false);
		outputSelection("");
		outputMessage(R.string.read_memory_read_message);

		resetActionResult();

		AsReaderP3xUMemoryBank bank = getMemBank();
		int offset = getMemOffset();
		int length = getMemLength();
		int time = getOperationTime();

		String password = getAccessPassword();
		try {
			Thread.sleep(50);
			getAsReaderP3xU().setAccessPassword(password);
		} catch (AsReaderP3xUException | InterruptedException e) {
			AsReaderP3xUASRLog.e(TAG, e, "ERROR. readMemory() - Failed to set password [%s]", password);
		}

		try {
			Thread.sleep(50);
			getAsReaderP3xU().setOperationTime(time);
		} catch (AsReaderP3xUException | InterruptedException e) {
			AsReaderP3xUASRLog.e(TAG, e, "ERROR. readMemory() - Failed to set operation time(%d)", time);
		}

		AsReaderP3xUResultCode res;
		if ((res = getAsReaderP3xU().readMemory(bank, offset, length)) != AsReaderP3xUResultCode.NoError) {
			AsReaderP3xUASRLog.e(TAG, "ERROR. readMemory() - Failed to start read memory(%s, %d, %d)", bank, offset, length);
			String msg = String.format(Locale.US, "%s : 0x%04X", res.toString(), res.getCode());
			Toast.makeText(this, msg, Toast.LENGTH_SHORT).show();
			enableActionWidgets(true);
			outputFailMessage(R.string.read_memory_fail_read_message);
			return;
		}
		
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. readMemory()");
	}

	// Initialize Reader
	@Override
	protected void initAsReaderP3xU() {
		super.initAsReaderP3xU();
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. initAsReaderP3xU()");
	}

	// Initialize Widgets
	@Override
	protected void initWidgets() {
		super.initWidgets();

		// Value ListView
		lstValue = (ListView) findViewById(R.id.read_memory);
		adpValue = new MemoryListAdapter(this);
		lstValue.setAdapter(adpValue);
		lstValue.setBackgroundColor(getColor(getApplicationContext(), R.color.message_background));
		//adpValue.setValue(0, "00000000000000000000000000000000");

		ScrollView scrlParams = (ScrollView) findViewById(R.id.read_params);
		scrlParams.setBackgroundColor(getColor(getApplicationContext(), R.color.message_background));
		
		// Length Spinner
		txtLength = (TextView) findViewById(R.id.length);
		txtLength.setOnClickListener(this);

		// Read Button
		btnRead = (Button) findViewById(R.id.action_read);
		btnRead.setOnClickListener(this);
		
		setMemLength(DEFAULT_MEM_LENGTH);

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. initWidgets()");
	}

	// Enable/Disable Action Widgets
	@Override
	protected void enableActionWidgets(boolean enabled) {
		super.enableActionWidgets(enabled);

		lstValue.setEnabled(isEnabledWidget(enabled));
		btnRead.setEnabled(isEnabledWidget(enabled, AsReaderP3xUActionState.ReadMemory));

		if (enabled) {
			switch (getAsReaderP3xU().getAction()) {
			case ReadMemory:
				btnRead.setText(R.string.action_stop);
				break;
			case Stop:
				btnRead.setText(R.string.action_read_memory);
				break;
			default:
				break;
			}
		}
		
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. enableActionWidgets(%s)", enabled);
	}

	@Override
	protected void clearWidgets() {
		super.clearWidgets();

		resetActionResult();
		adpValue.clear();

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. clearWidgets()");
	}

	protected void setMemLength(int length) {
		mLength = length;
		txtLength.setText(String.format(Locale.US, "%d WORD", mLength));
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. setMemLength(%d)", length);
	}

	protected int getMemLength() {
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. getMemLength() - [%d]", mLength);
		return mLength;
	}

	private void showMemLengthDialog() {
		if (!mIsEnabled)
			return;

		LinearLayout root = (LinearLayout) LinearLayout.inflate(this, R.layout.dialog_list_view, null);
		final ListView value = (ListView) root.findViewById(R.id.list);
		final SpinnerAdapter adapter = new SpinnerAdapter(this, android.R.layout.simple_list_item_single_choice);

		for(int n=0; n<=15; n++) {
			adapter.addItem(n, String.format(Locale.US, "%d WORD", n));
		}
		value.setAdapter(adapter);
		value.setChoiceMode(ListView.CHOICE_MODE_SINGLE);

		AlertDialog.Builder builder = new AlertDialog.Builder(this);
		builder.setTitle(R.string.length);
		builder.setView(root);
		builder.setPositiveButton(R.string.action_ok, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				int position;
				try {
					position = value.getCheckedItemPosition();
				} catch (Exception e) {
					position = DEFAULT_MEM_OFFSET;
				}
				setMemLength(position);
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showMemLengthDialog().$PositiveButton.onClick()");
			}
		});
		builder.setNegativeButton(R.string.action_cancel, new DialogInterface.OnClickListener() {

			@Override
			public void onClick(DialogInterface dialog, int which) {
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showMemLengthDialog().$NegativeButton.onClick()");
			}
		});
		builder.setCancelable(true);
		builder.setOnCancelListener(new DialogInterface.OnCancelListener() {

			@Override
			public void onCancel(DialogInterface dialog) {
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showMemLengthDialog().onCancel()");
			}
		});
		AlertDialog dialog = builder.create();
		dialog.setOnShowListener(new DialogInterface.OnShowListener() {

			@Override
			public void onShow(DialogInterface dialog) {
				int position = adapter.getPosition(mLength);
				value.setItemChecked(position, true);
				value.setSelectionFromTop(position, 0);
				AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showMemLengthDialog().onShow()");
			}
		});
		dialog.show();
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. showMemLengthDialog()");
	}
	
	private static class InitTask extends AsyncTask<Void, Void, Boolean> {

		private final WeakReference<ReadMemoryActivity> mActivity;

		InitTask(ReadMemoryActivity activity) {
			mActivity = new WeakReference<>(activity);
		}

		@Override
		protected void onPreExecute() {
			ReadMemoryActivity activity = mActivity.get();
			WaitDialog.show(activity, R.string.init_mask);
			super.onPreExecute();
		}

		@Override
		protected Boolean doInBackground(Void... params) {
			// Initialize Mask
			ReadMemoryActivity activity = mActivity.get();
			activity.initMask();
			return true;
		}
		
		@Override
		protected void onPostExecute(Boolean result) {
			ReadMemoryActivity activity = mActivity.get();

			activity.resetActionResult();
			activity.setMask(activity.getMask());
			// Enable action widget
			activity.enableActionWidgets(true);
			activity.outputMessage(R.string.read_memory_intro_message);
			WaitDialog.hide();
			super.onPostExecute(result);
		}
	}
}
