package com.asreader.support.andro.app.rfid.view.base;

import android.content.Intent;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import com.asreader.p3xu.diagnostics.AsReaderP3xUASRLog;
import com.asreader.p3xu.rfid.exception.AsReaderP3xUException;
import com.asreader.R;
import com.asreader.support.andro.app.rfid.dialog.WaitDialog;
import com.asreader.support.andro.app.rfid.util.ResUtil;
import com.asreader.support.andro.app.rfid.util.UserConfigs;
import com.asreader.support.andro.app.rfid.view.sub.EpcMaskActivity;

public abstract class RfidMaskActivity extends RfidPowerGainActivity implements OnClickListener {

	private static final String TAG = RfidMaskActivity.class.getSimpleName();

	public static final String KEY_MASK = "mask_value";

	protected Button btnMask;
	private String mMaskValue;

	public static UserConfigs mUserConfigs = null;

	public RfidMaskActivity() {
		mMaskValue = "";
	}

	@Override
	public void onClick(View v) {
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. onClick(%s)", ResUtil.getId(v.getId()));
		switch (v.getId()) {
			case R.id.mask:
				Intent intent = new Intent(this, EpcMaskActivity.class);
				startActivityForResult(intent, VIEW_MASK);
				break;
		}
		super.onClick(v);
	}

	@Override
	protected void initWidgets() {
		super.initWidgets();

		btnMask = (Button) findViewById(R.id.mask);
		btnMask.setOnClickListener(this);
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. initWidgets()");
	}

	@Override
	protected void enableActionWidgets(boolean enabled) {
		super.enableActionWidgets(enabled);

		if(btnMask != null)
			btnMask.setEnabled(isEnabledWidget(enabled) && mMaskValue.equals(""));
	}

	protected void initMask() {
		Intent intent = getIntent();
		if ((mMaskValue = intent.getStringExtra(KEY_MASK)) == null)
			mMaskValue = "";

		if (mMaskValue.equals("")) {
			WaitDialog.hide();
			return;
		}

		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. initMask() - [%s]", mMaskValue);
	}

	protected void exitMask() {

		if (mMaskValue.equals(""))
			return;

		try {
			getAsReaderP3xU().clearEpcMask();
			mUserConfigs = UserConfigs.getInstance(this.getApplicationContext());
			for (int i = 0; i < mUserConfigs.getEpcMaskCount(); i++) {
				getAsReaderP3xU().addEpcMask(mUserConfigs.getEpcMask(i));
			}
		} catch (AsReaderP3xUException e) {
			AsReaderP3xUASRLog.e(TAG, e, "ERROR. exitMask() - Failed to rollback use epc mask");
		}

		
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. exitMask()");
	}

	protected String getMask() {
		AsReaderP3xUASRLog.i(TAG, AsReaderP3xUASRLog.INFO, "INFO. getMask() - [%s]", mMaskValue);
		return mMaskValue;
	}
}
